@@ -31,6 +31,23 @@
 /**
  * SECTION:gstmixer
  * @short_description: Interface for elements that provide mixer operations
+ * @see_also: alsamixer, oss4mixer, sunaudiomixer
+ *
+ * Basic interface for hardware mixer controls.
+ *
+ * Applications rarely need to use this interface, it is provided mainly
+ * for system-level mixer applets and the like. Volume control in playback
+ * applications should be done using a <classname>volume</classname>
+ * element or, if available, using the <quote>volume</quote> property of
+ * the audio sink element used (as provided by <classname>pulsesink</classname>
+ * for example), or even better: just use the <classname>playbin2</classname>
+ * element's <quote>volume</quote> property.
+ *
+ * Usage: In order to use the <classname>GstMixer</classname> interface, the
+ * element needs to be at least in READY state (so that the element has opened
+ * the mixer device). Once the element has been set to READY state or higher,
+ * it can be cast to a <classname>GstMixer</classname> using the GST_MIXER
+ * macro (in C) and the mixer API can be used.
  */
 
 #ifndef GST_DISABLE_DEPRECATED
@@ -139,6 +156,8 @@ gst_mixer_class_init (GstMixerClass * klass)
  * it is allowed to only provide input elements in this list.
  *
  * Returns: A #GList consisting of zero or more #GstMixerTracks.
+ *          The list is owned by the #GstMixer instance and must not be freed
+ *          or modified.
  */
 
 const GList *
@@ -321,6 +340,24 @@ gst_mixer_get_option (GstMixer * mixer, GstMixerOptions * opts)
   return NULL;
 }
 
+/**
+ * gst_mixer_get_mixer_type:
+ * @mixer: The #GstMixer implementation
+ *
+ * Get the #GstMixerType of this mixer implementation.
+ *
+ * Returns: A the #GstMixerType.
+ *
+ * Since: 0.10.24
+ */
+GstMixerType
+gst_mixer_get_mixer_type (GstMixer * mixer)
+{
+  GstMixerClass *klass = GST_MIXER_GET_CLASS (mixer);
+
+  return klass->mixer_type;
+}
+
 /**
  * gst_mixer_get_mixer_flags:
  * @mixer: The #GstMixer implementation
@@ -595,7 +632,7 @@ gst_mixer_message_is_mixer_message (GstMessage * message)
  * message and return the GstMixerMessageType identifying which
  * type of notification it is.
  *
- * Returns: The type of the GstMixerMessage, or GST_MIXER_MESSAGE_NONE
+ * Returns: The type of the GstMixerMessage, or GST_MIXER_MESSAGE_INVALID
  * if the message is not a GstMixer notification.
  *
  * Since: 0.10.14
