@@ -24,12 +24,10 @@
 
 #include <gst/check/gstcheck.h>
 
-gboolean have_eos = FALSE;
-
 /* For ease of programming we use globals to keep refs for our floating
  * src and sink pads we create; otherwise we always have to do get_pad,
  * get_peer, and then remove references in every test function */
-GstPad *mysrcpad, *mysinkpad;
+static GstPad *mysrcpad, *mysinkpad;
 
 
 #define VIDEO_CAPS_TEMPLATE_STRING     \
@@ -48,19 +46,38 @@ GstPad *mysrcpad, *mysinkpad;
     "height = (int) 240, "              \
     "format = (fourcc) I420"
 
+#define VIDEO_CAPS_NEWSIZE_STRING       \
+    "video/x-raw-yuv, "                 \
+    "width = (int) 240, "               \
+    "height = (int) 120, "              \
+    "framerate = (fraction) 25/1 , "	\
+    "format = (fourcc) I420"
+
+#define VIDEO_CAPS_UNUSUAL_FRAMERATE    \
+    "video/x-raw-yuv, "                 \
+    "width = (int) 240, "               \
+    "height = (int) 120, "              \
+    "framerate = (fraction) 999/7 , "	\
+    "format = (fourcc) I420"
 
 static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS (VIDEO_CAPS_TEMPLATE_STRING)
     );
+static GstStaticPadTemplate downstreamsinktemplate =
+GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS (VIDEO_CAPS_STRING)
+    );
 static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS (VIDEO_CAPS_TEMPLATE_STRING)
     );
 
-void
+static void
 assert_videorate_stats (GstElement * videorate, gchar * reason,
     guint64 xin, guint64 xout, guint64 xdropped, guint64 xduplicated)
 {
@@ -84,22 +101,29 @@ G_STMT_START {                                                          \
   _assert_equals_uint64 (duplicated, xduplicated);
 }
 
-GstElement *
-setup_videorate ()
+static GstElement *
+setup_videorate_full (GstStaticPadTemplate * srctemplate,
+    GstStaticPadTemplate * sinktemplate)
 {
   GstElement *videorate;
 
   GST_DEBUG ("setup_videorate");
   videorate = gst_check_setup_element ("videorate");
-  mysrcpad = gst_check_setup_src_pad (videorate, &srctemplate, NULL);
-  mysinkpad = gst_check_setup_sink_pad (videorate, &sinktemplate, NULL);
+  mysrcpad = gst_check_setup_src_pad (videorate, srctemplate, NULL);
+  mysinkpad = gst_check_setup_sink_pad (videorate, sinktemplate, NULL);
   gst_pad_set_active (mysrcpad, TRUE);
   gst_pad_set_active (mysinkpad, TRUE);
 
   return videorate;
 }
 
-void
+static GstElement *
+setup_videorate (void)
+{
+  return setup_videorate_full (&srctemplate, &sinktemplate);
+}
+
+static void
 cleanup_videorate (GstElement * videorate)
 {
   GST_DEBUG ("cleanup_videorate");
@@ -132,6 +156,7 @@ GST_START_TEST (test_one)
   memset (GST_BUFFER_DATA (inbuffer), 0, 4);
   caps = gst_caps_from_string (VIDEO_CAPS_STRING);
   gst_buffer_set_caps (inbuffer, caps);
+  GST_BUFFER_TIMESTAMP (inbuffer) = 0;
   gst_caps_unref (caps);
   ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
 
@@ -153,6 +178,7 @@ GST_START_TEST (test_more)
   GstBuffer *first, *second, *third, *outbuffer;
   GList *l;
   GstCaps *caps;
+  GRand *rand;
 
   videorate = setup_videorate ();
   fail_unless (gst_element_set_state (videorate,
@@ -160,9 +186,15 @@ GST_START_TEST (test_more)
       "could not set to playing");
   assert_videorate_stats (videorate, "creation", 0, 0, 0, 0);
 
+  rand = g_rand_new ();
+
   /* first buffer */
   first = gst_buffer_new_and_alloc (4);
   GST_BUFFER_TIMESTAMP (first) = 0;
+  /* it shouldn't matter what the offsets are, videorate produces perfect
+     streams */
+  GST_BUFFER_OFFSET (first) = g_rand_int (rand);
+  GST_BUFFER_OFFSET_END (first) = g_rand_int (rand);
   memset (GST_BUFFER_DATA (first), 1, 4);
   caps = gst_caps_from_string (VIDEO_CAPS_STRING);
   gst_buffer_set_caps (first, caps);
@@ -180,6 +212,8 @@ GST_START_TEST (test_more)
   /* second buffer; inbetween second and third output frame's timestamp */
   second = gst_buffer_new_and_alloc (4);
   GST_BUFFER_TIMESTAMP (second) = GST_SECOND * 3 / 50;
+  GST_BUFFER_OFFSET (first) = g_rand_int (rand);
+  GST_BUFFER_OFFSET_END (first) = g_rand_int (rand);
   memset (GST_BUFFER_DATA (second), 2, 4);
   caps = gst_caps_from_string (VIDEO_CAPS_STRING);
   gst_buffer_set_caps (second, caps);
@@ -203,6 +237,8 @@ GST_START_TEST (test_more)
   /* third buffer */
   third = gst_buffer_new_and_alloc (4);
   GST_BUFFER_TIMESTAMP (third) = GST_SECOND * 12 / 50;
+  GST_BUFFER_OFFSET (first) = g_rand_int (rand);
+  GST_BUFFER_OFFSET_END (first) = g_rand_int (rand);
   memset (GST_BUFFER_DATA (third), 3, 4);
   caps = gst_caps_from_string (VIDEO_CAPS_STRING);
   gst_buffer_set_caps (third, caps);
@@ -222,17 +258,28 @@ GST_START_TEST (test_more)
   l = buffers;
   fail_unless_equals_uint64 (GST_BUFFER_TIMESTAMP (l->data), 0);
   fail_unless_equals_int (GST_BUFFER_DATA (l->data)[0], 1);
+  fail_unless_equals_uint64 (GST_BUFFER_OFFSET (l->data), 0);
+  fail_unless_equals_uint64 (GST_BUFFER_OFFSET_END (l->data), 1);
+
   l = g_list_next (l);
   fail_unless_equals_uint64 (GST_BUFFER_TIMESTAMP (l->data), GST_SECOND / 25);
   fail_unless_equals_int (GST_BUFFER_DATA (l->data)[0], 2);
+  fail_unless_equals_uint64 (GST_BUFFER_OFFSET (l->data), 1);
+  fail_unless_equals_uint64 (GST_BUFFER_OFFSET_END (l->data), 2);
+
   l = g_list_next (l);
   fail_unless_equals_uint64 (GST_BUFFER_TIMESTAMP (l->data),
       GST_SECOND * 2 / 25);
   fail_unless_equals_int (GST_BUFFER_DATA (l->data)[0], 2);
+  fail_unless_equals_uint64 (GST_BUFFER_OFFSET (l->data), 2);
+  fail_unless_equals_uint64 (GST_BUFFER_OFFSET_END (l->data), 3);
+
   l = g_list_next (l);
   fail_unless_equals_uint64 (GST_BUFFER_TIMESTAMP (l->data),
       GST_SECOND * 3 / 25);
   fail_unless_equals_int (GST_BUFFER_DATA (l->data)[0], 2);
+  fail_unless_equals_uint64 (GST_BUFFER_OFFSET (l->data), 3);
+  fail_unless_equals_uint64 (GST_BUFFER_OFFSET_END (l->data), 4);
 
   fail_unless_equals_int (g_list_length (buffers), 4);
   /* one held by us, three held by each output frame taken from the second */
@@ -247,6 +294,7 @@ GST_START_TEST (test_more)
   fail_unless_equals_int (g_list_length (buffers), 5);
 
   /* cleanup */
+  g_rand_free (rand);
   gst_buffer_unref (first);
   gst_buffer_unref (second);
   gst_buffer_unref (third);
@@ -488,8 +536,231 @@ GST_START_TEST (test_no_framerate)
 
 GST_END_TEST;
 
+/* This test outputs 2 buffers of same dimensions (320x240), then 1 buffer of 
+ * differing dimensions (240x120), and then another buffer of previous 
+ * dimensions (320x240) and checks that the 3 buffers output as a result have 
+ * correct caps (first 2 with 320x240 and 3rd with 240x120).
+ */
+GST_START_TEST (test_changing_size)
+{
+  GstElement *videorate;
+  GstBuffer *first;
+  GstBuffer *second;
+  GstBuffer *third;
+  GstBuffer *fourth;
+  GstBuffer *fifth;
+  GstBuffer *outbuf;
+  GstEvent *newsegment;
+  GstCaps *caps, *caps_newsize;
 
-Suite *
+  videorate = setup_videorate ();
+  fail_unless (gst_element_set_state (videorate,
+          GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
+      "could not set to playing");
+
+  newsegment = gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_TIME, 0, -1,
+      0);
+  fail_unless (gst_pad_push_event (mysrcpad, newsegment) == TRUE);
+
+  first = gst_buffer_new_and_alloc (4);
+  memset (GST_BUFFER_DATA (first), 0, 4);
+  caps = gst_caps_from_string (VIDEO_CAPS_STRING);
+  GST_BUFFER_TIMESTAMP (first) = 0;
+  gst_buffer_set_caps (first, caps);
+
+  GST_DEBUG ("pushing first buffer");
+  fail_unless (gst_pad_push (mysrcpad, first) == GST_FLOW_OK);
+
+  /* second buffer */
+  second = gst_buffer_new_and_alloc (4);
+  GST_BUFFER_TIMESTAMP (second) = GST_SECOND / 25;
+  memset (GST_BUFFER_DATA (second), 0, 4);
+  gst_buffer_set_caps (second, caps);
+
+  fail_unless (gst_pad_push (mysrcpad, second) == GST_FLOW_OK);
+  fail_unless_equals_int (g_list_length (buffers), 1);
+  outbuf = buffers->data;
+  /* first buffer should be output here */
+  fail_unless (gst_caps_is_equal (GST_BUFFER_CAPS (outbuf), caps));
+  fail_unless (GST_BUFFER_TIMESTAMP (outbuf) == 0);
+
+  /* third buffer with new size */
+  third = gst_buffer_new_and_alloc (4);
+  GST_BUFFER_TIMESTAMP (third) = 2 * GST_SECOND / 25;
+  memset (GST_BUFFER_DATA (third), 0, 4);
+  caps_newsize = gst_caps_from_string (VIDEO_CAPS_NEWSIZE_STRING);
+  gst_buffer_set_caps (third, caps_newsize);
+
+  fail_unless (gst_pad_push (mysrcpad, third) == GST_FLOW_OK);
+  /* new caps flushed the internal state, no new output yet */
+  fail_unless_equals_int (g_list_length (buffers), 1);
+  outbuf = g_list_last (buffers)->data;
+  /* first buffer should be output here */
+  fail_unless (gst_caps_is_equal (GST_BUFFER_CAPS (outbuf), caps));
+  fail_unless (GST_BUFFER_TIMESTAMP (outbuf) == 0);
+
+  /* fourth buffer with original size */
+  fourth = gst_buffer_new_and_alloc (4);
+  GST_BUFFER_TIMESTAMP (fourth) = 3 * GST_SECOND / 25;
+  memset (GST_BUFFER_DATA (fourth), 0, 4);
+  gst_buffer_set_caps (fourth, caps);
+
+  fail_unless (gst_pad_push (mysrcpad, fourth) == GST_FLOW_OK);
+  fail_unless_equals_int (g_list_length (buffers), 1);
+
+  /* fifth buffer with original size */
+  fifth = gst_buffer_new_and_alloc (4);
+  GST_BUFFER_TIMESTAMP (fifth) = 4 * GST_SECOND / 25;
+  memset (GST_BUFFER_DATA (fifth), 0, 4);
+  gst_buffer_set_caps (fifth, caps);
+
+  fail_unless (gst_pad_push (mysrcpad, fifth) == GST_FLOW_OK);
+  /* all four missing buffers here, dups of fourth buffer */
+  fail_unless_equals_int (g_list_length (buffers), 4);
+  outbuf = g_list_last (buffers)->data;
+  /* third buffer should be output here */
+  fail_unless (GST_BUFFER_TIMESTAMP (outbuf) == 3 * GST_SECOND / 25);
+  fail_unless (gst_caps_is_equal (GST_BUFFER_CAPS (outbuf), caps));
+
+  gst_caps_unref (caps);
+  gst_caps_unref (caps_newsize);
+  cleanup_videorate (videorate);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_non_ok_flow)
+{
+  GstElement *videorate;
+  GstClockTime ts;
+  GstBuffer *buf;
+  GstCaps *caps;
+
+  videorate = setup_videorate ();
+  fail_unless (gst_element_set_state (videorate,
+          GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
+      "could not set to playing");
+
+  buf = gst_buffer_new_and_alloc (4);
+  memset (GST_BUFFER_DATA (buf), 0, 4);
+  caps = gst_caps_from_string (VIDEO_CAPS_STRING);
+  gst_buffer_set_caps (buf, caps);
+  gst_caps_unref (caps);
+  ASSERT_BUFFER_REFCOUNT (buf, "inbuffer", 1);
+
+  /* push a few 'normal' buffers */
+  for (ts = 0; ts < 100 * GST_SECOND; ts += GST_SECOND / 33) {
+    GstBuffer *inbuf;
+
+    inbuf = gst_buffer_copy (buf);
+    GST_BUFFER_TIMESTAMP (inbuf) = ts;
+
+    fail_unless_equals_int (gst_pad_push (mysrcpad, inbuf), GST_FLOW_OK);
+  }
+
+  /* we should have buffers according to the output framerate of 25/1 */
+  fail_unless_equals_int (g_list_length (buffers), 100 * 25);
+
+  /* now deactivate pad so we get a WRONG_STATE flow return */
+  gst_pad_set_active (mysinkpad, FALSE);
+
+  /* push buffer on deactivated pad */
+  fail_unless (gst_buffer_is_metadata_writable (buf));
+  GST_BUFFER_TIMESTAMP (buf) = ts;
+
+  /* pushing gives away our reference */
+  fail_unless_equals_int (gst_pad_push (mysrcpad, buf), GST_FLOW_WRONG_STATE);
+
+  /* cleanup */
+  cleanup_videorate (videorate);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_upstream_caps_nego)
+{
+  GstElement *videorate;
+  GstPad *videorate_pad;
+  GstCaps *expected_caps;
+  GstCaps *caps;
+  GstStructure *structure;
+
+  videorate = setup_videorate_full (&srctemplate, &downstreamsinktemplate);
+  fail_unless (gst_element_set_state (videorate,
+          GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
+      "could not set to playing");
+
+  videorate_pad = gst_element_get_pad (videorate, "sink");
+  caps = gst_pad_get_caps (videorate_pad);
+
+  /* assemble the expected caps */
+  structure = gst_structure_from_string (VIDEO_CAPS_STRING, NULL);
+  expected_caps = gst_caps_new_empty ();
+  gst_caps_append_structure (expected_caps, structure);
+  structure = gst_structure_copy (structure);
+  gst_structure_set (structure, "framerate", GST_TYPE_FRACTION_RANGE,
+      0, 1, G_MAXINT, 1, NULL);
+  gst_caps_append_structure (expected_caps, structure);
+
+  fail_unless (gst_caps_is_equal (expected_caps, caps));
+  gst_caps_unref (caps);
+  gst_caps_unref (expected_caps);
+  gst_object_unref (videorate_pad);
+
+  /* cleanup */
+  cleanup_videorate (videorate);
+}
+
+GST_END_TEST;
+
+
+GST_START_TEST (test_selected_caps)
+{
+  GstElement *videorate;
+  GstElement *pipeline;
+  GstBus *bus;
+  GstMessage *msg;
+
+  GstPad *videorate_pad;
+  GstCaps *caps = NULL;
+  GstCaps *expected_caps = NULL;
+
+  pipeline = gst_parse_launch ("videotestsrc num-buffers=1 ! "
+      "! identity ! videorate name=videorate0 ! " VIDEO_CAPS_UNUSUAL_FRAMERATE
+      " ! fakesink", NULL);
+  fail_if (pipeline == NULL);
+  videorate = gst_bin_get_by_name (GST_BIN (pipeline), "videorate0");
+  fail_if (videorate == NULL);
+  bus = gst_pipeline_get_bus (GST_PIPELINE (pipeline));
+
+  fail_if (gst_element_set_state (pipeline,
+          GST_STATE_PLAYING) == GST_STATE_CHANGE_FAILURE,
+      "could not set to playing");
+
+  msg = gst_bus_timed_pop_filtered (bus, GST_CLOCK_TIME_NONE,
+      GST_MESSAGE_EOS | GST_MESSAGE_ERROR);
+  fail_if (msg == NULL || GST_MESSAGE_TYPE (msg) == GST_MESSAGE_ERROR);
+
+  videorate_pad = gst_element_get_pad (videorate, "sink");
+  g_object_get (videorate_pad, "caps", &caps, NULL);
+  expected_caps = gst_caps_from_string (VIDEO_CAPS_UNUSUAL_FRAMERATE);
+
+  fail_unless (gst_caps_is_equal (expected_caps, caps));
+
+  /* cleanup */
+  gst_object_unref (bus);
+  gst_message_unref (msg);
+  gst_caps_unref (caps);
+  gst_caps_unref (expected_caps);
+  gst_object_unref (videorate_pad);
+  gst_element_set_state (pipeline, GST_STATE_NULL);
+  gst_element_get_state (pipeline, NULL, NULL, GST_CLOCK_TIME_NONE);
+  gst_object_unref (pipeline);
+}
+
+GST_END_TEST;
+
+static Suite *
 videorate_suite (void)
 {
   Suite *s = suite_create ("videorate");
@@ -501,23 +772,12 @@ videorate_suite (void)
   tcase_add_test (tc_chain, test_wrong_order_from_zero);
   tcase_add_test (tc_chain, test_wrong_order);
   tcase_add_test (tc_chain, test_no_framerate);
+  tcase_add_test (tc_chain, test_changing_size);
+  tcase_add_test (tc_chain, test_non_ok_flow);
+  tcase_add_test (tc_chain, test_upstream_caps_nego);
+  tcase_add_test (tc_chain, test_selected_caps);
 
   return s;
 }
 
-int
-main (int argc, char **argv)
-{
-  int nf;
-
-  Suite *s = videorate_suite ();
-  SRunner *sr = srunner_create (s);
-
-  gst_check_init (&argc, &argv);
-
-  srunner_run_all (sr, CK_NORMAL);
-  nf = srunner_ntests_failed (sr);
-  srunner_free (sr);
-
-  return nf;
-}
+GST_CHECK_MAIN (videorate)
