@@ -2,7 +2,7 @@
  *
  * unit tests for the tag support library
  *
- * Copyright (C) 2006 Tim-Philipp Müller <tim centricular net>
+ * Copyright (C) 2006-2009 Tim-Philipp Müller <tim centricular net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -165,6 +165,18 @@ GST_END_TEST;
     fail_unless_equals_int (___n, num);                                    \
   }
 
+#define MATCH_DOUBLE(p1, p2) ((p1 < p2 + 1e-6) && (p2 < p1 + 1e-6))
+#define ASSERT_TAG_LIST_HAS_DOUBLE(list,field,d)                           \
+  {                                                                        \
+    gdouble ___d;                                                          \
+                                                                           \
+    fail_unless (gst_tag_list_get_tag_size (list,field) > 0);              \
+    fail_unless (gst_tag_list_get_tag_size (list,field) == 1);             \
+    fail_unless (gst_tag_list_get_double_index (list, field, 0, &___d));   \
+    fail_unless (MATCH_DOUBLE (d, ___d),                                   \
+        "%f does not match expected %f", ___d, d);                         \
+  }
+
 GST_START_TEST (test_muscibrainz_tag_registration)
 {
   GstTagList *list;
@@ -179,6 +191,7 @@ GST_START_TEST (test_muscibrainz_tag_registration)
   gst_vorbis_tag_add (list, "MUSICBRAINZ_ALBUMID", "345678");
   gst_vorbis_tag_add (list, "MUSICBRAINZ_ALBUMARTISTID", "4567890");
   gst_vorbis_tag_add (list, "MUSICBRAINZ_TRMID", "5678901");
+  /* MUSICBRAINZ_SORTNAME = GST_TAG_ARTIST_SORTNAME now */
   gst_vorbis_tag_add (list, "MUSICBRAINZ_SORTNAME", "Five, 678901");
 
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_MUSICBRAINZ_TRACKID, "123456");
@@ -187,8 +200,7 @@ GST_START_TEST (test_muscibrainz_tag_registration)
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_MUSICBRAINZ_ALBUMARTISTID,
       "4567890");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_MUSICBRAINZ_TRMID, "5678901");
-  ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_MUSICBRAINZ_SORTNAME,
-      "Five, 678901");
+  ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_ARTIST_SORTNAME, "Five, 678901");
 
   gst_tag_list_free (list);
 }
@@ -246,6 +258,9 @@ GST_START_TEST (test_vorbis_tags)
   gst_vorbis_tag_add (list, "COPYRIGHT", "Copyfoo");
   gst_vorbis_tag_add (list, "DESCRIPTION", "Descoo");
   gst_vorbis_tag_add (list, "LICENSE", "Licoo");
+  gst_vorbis_tag_add (list, "LICENSE",
+      "http://creativecommons.org/licenses/by/3.0/");
+  gst_vorbis_tag_add (list, "LOCATION", "Bristol, UK");
   gst_vorbis_tag_add (list, "ORGANIZATION", "Orgoo");
   gst_vorbis_tag_add (list, "GENRE", "Goo");
   gst_vorbis_tag_add (list, "CONTACT", "Coo");
@@ -265,13 +280,15 @@ GST_START_TEST (test_vorbis_tags)
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_COPYRIGHT, "Copyfoo");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_DESCRIPTION, "Descoo");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_LICENSE, "Licoo");
+  ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_LICENSE_URI,
+      "http://creativecommons.org/licenses/by/3.0/");
+  ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_GEO_LOCATION_NAME, "Bristol, UK");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_ORGANIZATION, "Orgoo");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_GENRE, "Goo");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_CONTACT, "Coo");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_COMMENT,
       "Peroxysulfid stroodles the brain");
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_COMMENT, "Stroodle is good");
-
   ASSERT_TAG_LIST_HAS_UINT (list, GST_TAG_TRACK_NUMBER, 5);
   ASSERT_TAG_LIST_HAS_UINT (list, GST_TAG_TRACK_COUNT, 77);
   ASSERT_TAG_LIST_HAS_UINT (list, GST_TAG_ALBUM_VOLUME_NUMBER, 1);
@@ -297,21 +314,21 @@ GST_START_TEST (test_vorbis_tags)
   ASSERT_TAG_LIST_HAS_STRING (list, GST_TAG_EXTENDED_COMMENT,
       "RuBuWuHash=1337BA42F91");
 
-#if 0
-  /* TODO: test these as well */
-  {
-  GST_TAG_TRACK_GAIN, "REPLAYGAIN_TRACK_GAIN"}
-  , {
-  GST_TAG_TRACK_PEAK, "REPLAYGAIN_TRACK_PEAK"}
-  , {
-  GST_TAG_ALBUM_GAIN, "REPLAYGAIN_ALBUM_GAIN"}
-  , {
-  GST_TAG_ALBUM_PEAK, "REPLAYGAIN_ALBUM_PEAK"}
-  , {
-  GST_TAG_LANGUAGE_CODE, "LANGUAGE"}
-  ,
-#endif
-      /* make sure we can convert back and forth without loss */
+  gst_vorbis_tag_add (list, "REPLAYGAIN_REFERENCE_LOUDNESS", "89.");
+  ASSERT_TAG_LIST_HAS_DOUBLE (list, GST_TAG_REFERENCE_LEVEL, 89.);
+  gst_vorbis_tag_add (list, "REPLAYGAIN_TRACK_GAIN", "+12.36");
+  ASSERT_TAG_LIST_HAS_DOUBLE (list, GST_TAG_TRACK_GAIN, +12.36);
+  gst_vorbis_tag_add (list, "REPLAYGAIN_TRACK_PEAK", "0.96349");
+  ASSERT_TAG_LIST_HAS_DOUBLE (list, GST_TAG_TRACK_PEAK, 0.96349);
+  gst_vorbis_tag_add (list, "REPLAYGAIN_ALBUM_GAIN", "+10.12");
+  ASSERT_TAG_LIST_HAS_DOUBLE (list, GST_TAG_ALBUM_GAIN, +10.12);
+  /* now check that we can parse floating point numbers with any separator
+   * (',' or '.') regardless of the current locale */
+  gst_vorbis_tag_add (list, "REPLAYGAIN_ALBUM_PEAK", "0,98107");
+  ASSERT_TAG_LIST_HAS_DOUBLE (list, GST_TAG_ALBUM_PEAK, 0.98107);
+  gst_vorbis_tag_add (list, "LICENSE", "http://foo.com/license-1.html");
+
+  /* make sure we can convert back and forth without loss */
   {
     GstTagList *new_list, *even_newer_list;
     GstBuffer *buf, *buf2;
@@ -497,6 +514,54 @@ GST_START_TEST (test_vorbis_tags)
 
     gst_buffer_unref (buf);
   }
+
+  /* check date with time */
+  {
+    GDate *date = NULL;
+
+    list = gst_tag_list_new ();
+    gst_vorbis_tag_add (list, "DATE", "2006-09-25 22:02:38");
+
+    fail_unless (gst_tag_list_get_date_index (list, GST_TAG_DATE, 0, &date));
+    fail_unless (date != NULL);
+    fail_unless (g_date_get_day (date) == 25);
+    fail_unless (g_date_get_month (date) == G_DATE_SEPTEMBER);
+    fail_unless (g_date_get_year (date) == 2006);
+
+    g_date_free (date);
+    gst_tag_list_free (list);
+  }
+
+  /* check date with month/day of 00-00 */
+  {
+    GDate *date = NULL;
+
+    list = gst_tag_list_new ();
+    gst_vorbis_tag_add (list, "DATE", "1992-00-00");
+
+    fail_unless (gst_tag_list_get_date_index (list, GST_TAG_DATE, 0, &date));
+    fail_unless (date != NULL);
+    fail_unless (g_date_get_year (date) == 1992);
+
+    g_date_free (date);
+    gst_tag_list_free (list);
+  }
+
+  /* check date with valid month, but day of 00 */
+  {
+    GDate *date = NULL;
+
+    list = gst_tag_list_new ();
+    gst_vorbis_tag_add (list, "DATE", "1992-05-00");
+
+    fail_unless (gst_tag_list_get_date_index (list, GST_TAG_DATE, 0, &date));
+    fail_unless (date != NULL);
+    fail_unless (g_date_get_year (date) == 1992);
+    fail_unless (g_date_get_month (date) == G_DATE_MAY);
+
+    g_date_free (date);
+    gst_tag_list_free (list);
+  }
 }
 
 GST_END_TEST;
@@ -522,6 +587,8 @@ GST_START_TEST (test_id3_tags)
   fail_unless (gst_tag_from_id3_tag ("TALB") != NULL);
   ASSERT_CRITICAL (gst_tag_from_id3_tag (NULL));
   fail_unless (gst_tag_from_id3_tag ("R2D2") == NULL);
+  fail_unless_equals_string (gst_tag_from_id3_tag ("WCOP"),
+      GST_TAG_COPYRIGHT_URI);
 
   /* gst_tag_from_id3_user_tag */
   ASSERT_CRITICAL (gst_tag_from_id3_user_tag (NULL, "foo"));
@@ -532,7 +599,8 @@ GST_START_TEST (test_id3_tags)
   ASSERT_CRITICAL (gst_tag_to_id3_tag (NULL));
   fail_unless (gst_tag_to_id3_tag ("R2D2") == NULL);
   fail_unless (gst_tag_to_id3_tag (GST_TAG_ARTIST) != NULL);
-
+  fail_unless_equals_string (gst_tag_to_id3_tag (GST_TAG_COPYRIGHT_URI),
+      "WCOP");
 
   fail_unless (GST_TYPE_TAG_IMAGE_TYPE != 0);
   fail_unless (g_type_name (GST_TYPE_TAG_IMAGE_TYPE) != NULL);
@@ -540,6 +608,147 @@ GST_START_TEST (test_id3_tags)
 
 GST_END_TEST;
 
+
+GST_START_TEST (test_id3v1_utf8_tag)
+{
+  const guint8 id3v1[128] = {
+    /* marker */
+    'T', 'A', 'G',
+    /* title (30 bytes) */
+    'D', 0xc3, 0xad, 'v', 'k', 'a', ' ', 's',
+    ' ', 'p', 'e', 'r', 'l', 'a', 'm', 'i',
+    ' ', 'v', 'e', ' ', 'v', 'l', 'a', 's',
+    'e', 'c', 'h', 0, 0, 0,
+    /* artist (30 bytes) */
+    'A', 'l', 'e', 0xc5, 0xa1, ' ', 'B', 'r', 'i', 'c', 'h', 't', 'a',
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    /* album (30 bytes) */
+    'B', 'e', 's', 't', ' ', 'o', 'f', ' ', '(', 'P', 'r', 'o', 's', 't',
+    0xc4, 0x9b, ' ', 0xc3, 0xba, 0xc5, 0xbe, 'a', 's', 'n', 0xc3, 0xbd, ')',
+    0, 0, 0,
+    /* year (4 bytes) */
+    '2', '0', '0', '0',
+    /* comment (28 bytes) */
+    '-', '-', '-', ' ', 0xc4, 0x8d, 'e', 's', 'k', 0xc3, 0xa9, ' ', 'p',
+    0xc3, 0xad, 's', 'n', 'i', 0xc4, 0x8d, 'k', 'y', ' ', '-', '-', '-',
+    0, 0,
+    /* track number */
+    0, 0,
+    /* genre */
+    0x11
+  };
+  GstTagList *tags;
+  GDate *d;
+  gchar *s;
+
+  /* set this, to make sure UTF-8 strings are really interpreted properly
+   * as UTF-8, regardless of the locale set */
+  g_setenv ("GST_ID3V1_TAG_ENCODING", "WINDOWS-1250", TRUE);
+
+  tags = gst_tag_list_new_from_id3v1 (id3v1);
+  fail_unless (tags != NULL);
+
+  GST_LOG ("Got tags: %" GST_PTR_FORMAT, tags);
+
+  s = NULL;
+  fail_unless (gst_tag_list_get_string (tags, GST_TAG_TITLE, &s));
+  fail_unless (s != NULL);
+  fail_unless_equals_string (s, "Dívka s perlami ve vlasech");
+  g_free (s);
+
+  s = NULL;
+  fail_unless (gst_tag_list_get_string (tags, GST_TAG_ARTIST, &s));
+  fail_unless (s != NULL);
+  fail_unless_equals_string (s, "Aleš Brichta");
+  g_free (s);
+
+  s = NULL;
+  fail_unless (gst_tag_list_get_string (tags, GST_TAG_ALBUM, &s));
+  fail_unless (s != NULL);
+  fail_unless_equals_string (s, "Best of (Prostě úžasný)");
+  g_free (s);
+
+  d = NULL;
+  fail_unless (gst_tag_list_get_date (tags, GST_TAG_DATE, &d));
+  fail_unless (d != NULL);
+  fail_unless_equals_int (g_date_get_year (d), 2000);
+  g_date_free (d);
+  d = NULL;
+
+  gst_tag_list_free (tags);
+
+  g_unsetenv ("GST_ID3V1_TAG_ENCODING");
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_language_utils)
+{
+  gchar **lang_codes, **c;
+
+#define ASSERT_STRINGS_EQUAL fail_unless_equals_string
+
+  lang_codes = gst_tag_get_language_codes ();
+  fail_unless (lang_codes != NULL);
+  fail_unless (*lang_codes != NULL);
+
+  for (c = lang_codes; c != NULL && *c != NULL; ++c) {
+    const gchar *lang_name, *c1, *c2t, *c2b;
+
+    lang_name = gst_tag_get_language_name (*c);
+    fail_unless (lang_name != NULL);
+    fail_unless (g_utf8_validate (lang_name, -1, NULL));
+
+    c1 = gst_tag_get_language_code_iso_639_1 (*c);
+    fail_unless (c1 != NULL);
+    fail_unless (g_utf8_validate (c1, -1, NULL));
+
+    c2t = gst_tag_get_language_code_iso_639_2T (*c);
+    fail_unless (c2t != NULL);
+    fail_unless (g_utf8_validate (c2t, -1, NULL));
+
+    c2b = gst_tag_get_language_code_iso_639_2B (*c);
+    fail_unless (c2b != NULL);
+    fail_unless (g_utf8_validate (c2b, -1, NULL));
+
+    ASSERT_STRINGS_EQUAL (gst_tag_get_language_code_iso_639_1 (*c), *c);
+    ASSERT_STRINGS_EQUAL (gst_tag_get_language_code_iso_639_1 (c2t), *c);
+    ASSERT_STRINGS_EQUAL (gst_tag_get_language_code_iso_639_1 (c2b), *c);
+
+    GST_DEBUG ("[%s] %s %s %s : %s\n", *c, c1, c2t, c2b, lang_name);
+
+  }
+  g_strfreev (lang_codes);
+
+  fail_unless (gst_tag_get_language_name ("de") != NULL);
+  fail_unless (gst_tag_get_language_name ("deu") != NULL);
+  fail_unless (gst_tag_get_language_name ("ger") != NULL);
+  fail_unless_equals_string (gst_tag_get_language_name ("deu"),
+      gst_tag_get_language_name ("ger"));
+  fail_unless_equals_string (gst_tag_get_language_name ("de"),
+      gst_tag_get_language_name ("ger"));
+  fail_unless (gst_tag_get_language_name ("de") !=
+      gst_tag_get_language_name ("fr"));
+
+  ASSERT_STRINGS_EQUAL (gst_tag_get_language_code ("deu"), "de");
+  ASSERT_STRINGS_EQUAL (gst_tag_get_language_code ("de"), "de");
+  ASSERT_STRINGS_EQUAL (gst_tag_get_language_code ("ger"), "de");
+
+  ASSERT_STRINGS_EQUAL (gst_tag_get_language_code_iso_639_1 ("deu"), "de");
+  ASSERT_STRINGS_EQUAL (gst_tag_get_language_code_iso_639_1 ("de"), "de");
+  ASSERT_STRINGS_EQUAL (gst_tag_get_language_code_iso_639_1 ("ger"), "de");
+
+  ASSERT_STRINGS_EQUAL (gst_tag_get_language_code_iso_639_2T ("de"), "deu");
+  ASSERT_STRINGS_EQUAL (gst_tag_get_language_code_iso_639_2T ("deu"), "deu");
+  ASSERT_STRINGS_EQUAL (gst_tag_get_language_code_iso_639_2T ("ger"), "deu");
+
+  ASSERT_STRINGS_EQUAL (gst_tag_get_language_code_iso_639_2B ("de"), "ger");
+  ASSERT_STRINGS_EQUAL (gst_tag_get_language_code_iso_639_2B ("deu"), "ger");
+  ASSERT_STRINGS_EQUAL (gst_tag_get_language_code_iso_639_2B ("ger"), "ger");
+}
+
+GST_END_TEST;
+
 static Suite *
 tag_suite (void)
 {
@@ -551,6 +760,8 @@ tag_suite (void)
   tcase_add_test (tc_chain, test_parse_extended_comment);
   tcase_add_test (tc_chain, test_vorbis_tags);
   tcase_add_test (tc_chain, test_id3_tags);
+  tcase_add_test (tc_chain, test_id3v1_utf8_tag);
+  tcase_add_test (tc_chain, test_language_utils);
   return s;
 }
 
